
	/************************************************************************
	 *
	 *  Usage:   string.pl   datafile   min_period  max_period  num_period
	 * 
	 *  Calculate the string-length statistic for a variable star light curve.
	 *  Expect data in the form
	 * 
	 *               JD      mag
	 *               JD      mag
	 *               JD      mag
	 * 
	 *  Iterate between the min and max supplied periods, taking num_period
	 *  steps.  For each candidate period, calculate the phases for each
	 *  datum, then sort by phase and calculate the string length in
	 *  phase-mag space.
	 * 
	 * 
	 *  MWR 10/23/2003
	 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#undef DEBUG 
#define MAXDATA 10000
#define LINELEN 256

struct s_phase {
	double jd;
	double mag;
	double phase;
};

	/* we hold original data in these arrays */
int ndata = 0;
struct s_phase data[MAXDATA];


typedef int (*PFI)();

static int compare_double (struct s_phase *p1, struct s_phase *p2);





int
main
	(
	int argc,
	char *argv[]
	)
{
	char datafile[LINELEN + 1];
	char line[LINELEN + 1];
	int num_period;
	int i, j, k;
	double min_period, max_period;
	double dperiod, period;
	double jd, mag;
	double phase;
	double x1, x2, y1, y2;
	double dist;
	double best_period;
	double length, min_length;
	FILE *fp;

	if (argc != 5) {
		fprintf(stderr, "usage: string datafile min max num\n");
		exit(1);
	}
	strcpy(datafile, argv[1]);
	sscanf(argv[2], "%lf", &min_period);
	sscanf(argv[3], "%lf", &max_period);
	sscanf(argv[4], "%d", &num_period);

#ifdef DEBUG
	printf("  datafile %s  min %lf max %lf num %d \n", 
			datafile, min_period, max_period, num_period);
#endif


	min_length = 1.0e6;
	best_period = -99.0;


	if ((fp = fopen(datafile, "r")) == NULL) {
		fprintf(stderr, "can't open file %s for reading\n", datafile);
		exit(1);
	}

	ndata = 0;
	while (fgets(line, LINELEN, fp) != NULL) {
		if (line[0] == '#') {
			continue;
		}

		if (sscanf(line, "%lf %lf", &jd, &mag) != 2) {
			fprintf(stderr, "bad format for line ..%s..\n", line);
			exit(1);
		}
#ifdef DEBUG
		printf("  jd %12.5f  mag %6.3f \n", jd, mag);
#endif
		data[ndata].jd = jd;
		data[ndata].mag = mag;
		ndata++;
		

	}
	fclose(fp);
  

	/*
	 * now, we try out one period at a time ... 
	 */
	dperiod = (max_period - min_period)/((double) num_period);
	for (period = min_period; period <= max_period; period += dperiod) {


#ifdef DEBUG
		printf (" period is %lf \n", period);
#endif

		/* calculate the phase for each point */
		for (i = 0; i < ndata; i++) {
			phase = (data[i].jd/period) - floor(data[i].jd/period);
			data[i].phase = phase;
#ifdef DEBUG
			printf("    i %5d  jd %12.5f  phase %5.3f  mag %6.3f \n",
						i, data[i].jd, phase, data[i].mag);
#endif
		}

		/* now sort the structure by phase */
		qsort((char *) data, ndata, sizeof(struct s_phase), (PFI) compare_double);

#ifdef DEBUG
		/* verify that the data are now sorted by phase */
		for (i = 0; i < ndata; i++) {
			printf("    i %5d  jd %12.5f  phase %5.3f  mag %6.3f \n",
						i, data[i].jd, data[i].phase, data[i].mag);
		}
#endif

		/* calculate the string length for this choice of period */
		length = 0;
		for (i = 1; i < ndata; i++) {
			x1 = data[i - 1].phase;
			x2 = data[i].phase;
			y1 = data[i - 1].mag;
			y2 = data[i].mag;
			dist = sqrt((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2));
			length += dist;
#ifdef DEBUG
			printf("  %6.2f %6.2f  %6.3f %6.3f  dist %6.2f  length %8.2f \n",
					x1, x2, y1, y2, dist, length);
#endif
		}

		/* print out the main output: a line with period and total length */
		printf ("%7.3f  %8.3f \n", period, length);

		/* is this best so far? */
		if (length < min_length) {
			min_length = length;
			best_period = period;
		}
	}

	/* now, re-calculate phases for best period */
	period = best_period;
	for (i = 0; i < ndata; i++) {
		phase = (data[i].jd/period) - floor(data[i].jd/period);
		data[i].phase = phase;
#ifdef DEBUG
		printf("    i %5d  jd %12.5f  phase %5.3f  mag %6.3f \n",
					i, data[i].jd, phase, data[i].mag);
#endif
	}
	/* sort the structure by phase */
	qsort((char *) data, ndata, sizeof(struct s_phase), (PFI) compare_double);
	/* and print out the phased data nicely */
	printf("# best period %7.3f  length %8.3f \n", best_period, min_length);
	for (i = 0; i < ndata; i++) {
		printf("     %5d   %12.5f   %5.3f   %6.3f \n",
					i, data[i].jd, data[i].phase, data[i].mag);
	}



	exit(0);
}



/************************************************************************
 *  * 
 *   *
 *    * ROUTINE: compare_double
 *     *
 *      * DESCRIPTION:
 *       * Given pointers to two double numbers, return the comparison.
 *        * Used by "iter_trans"
 *         *
 *          * RETURN:
 *           *    1                  if first double is larger than second
 *            *    0                  if the two are equal
 *             *   -1                  if first double is smaller than second
 *              *
 *               * </AUTO>
 */

static int
compare_double
   (
    struct s_phase *f1,                 /* I: compare size of FIRST double value */
    struct s_phase *f2                  /* I:  ... with SECOND double value  */
   )
{
   if (f1->phase > f2->phase) {
      return(1);
   }
	if (f1->phase < f2->phase) {
      return(-1);
   }
   return(0);
}



